function [g1, g2] = lfmXlfmKernGradient(lfmKern1, lfmKern2, t1, t2, covGrad)

% LFMXLFMKERNGRADIENT Compute a cross gradient between two LFM kernels.
%
%	Description:
%
%	[G1, G2] = LFMXLFMKERNGRADIENT(LFMKERN1, LFMKERN2, T1, COVGRAD)
%	computes cross gradient of parameters of a cross kernel between two
%	lfm kernels for the multiple output kernel.
%	 Returns:
%	  G1 - gradient of the parameters of the first kernel, for ordering
%	   see lfmKernExtractParam.
%	  G2 - gradient of the parameters of the second kernel, for ordering
%	   see lfmKernExtractParam.
%	 Arguments:
%	  LFMKERN1 - the kernel structure associated with the first LFM
%	   kernel.
%	  LFMKERN2 - the kernel structure associated with the second LFM
%	   kernel.
%	  T1 - inputs for which kernel is to be computed.
%	  COVGRAD - gradient of the objective function with respect to the
%	   elements of the cross kernel matrix.
%
%	[G1, G2] = LFMXLFMKERNGRADIENT(LFMKERN1, LFMKERN2, T1, T2, COVGRAD)
%	computes cross kernel terms between two LFM kernels for the multiple
%	output kernel.
%	 Returns:
%	  G1 - gradient of the parameters of the first kernel, for ordering
%	   see lfmKernExtractParam.
%	  G2 - gradient of the parameters of the second kernel, for ordering
%	   see lfmKernExtractParam.
%	 Arguments:
%	  LFMKERN1 - the kernel structure associated with the first LFM
%	   kernel.
%	  LFMKERN2 - the kernel structure associated with the second LFM
%	   kernel.
%	  T1 - row inputs for which kernel is to be computed.
%	  T2 - column inputs for which kernel is to be computed.
%	  COVGRAD - gradient of the objective function with respect to the
%	   elements of the cross kernel matrix.
%	
%	
%	
%
%	See also
%	MULTIKERNPARAMINIT, MULTIKERNCOMPUTE, LFMKERNPARAMINIT, LFMKERNEXTRACTPARAM


%	Copyright (c) 2007, 2008 David Luengo


%	With modifications by Neil D. Lawrence 2007


%	With modifications by Mauricio Alvarez 2008
% 	lfmXlfmKernGradient.m SVN version 11
% 	last update 2008-06-05T18:41:50.000000Z

  
if nargin < 5
  covGrad = t2;
  t2 = t1;
end
if size(t1, 2) > 1 | size(t2, 2) > 1
  error('Input can only have one column');
end
if lfmKern1.inverseWidth ~= lfmKern2.inverseWidth
  error('Kernels cannot be cross combined if they have different inverse widths.')
end

%Kxx = lfmXlfmKernCompute(lfmKern1, lfmKern2, t1, t2);

% Check whether lfmKern1 is the same as lfmKern2 (autocorrelation) or not.
% For the time being we consider that both kernels are equal if all their
% parameters are the same

par1 = kernExtractParam(lfmKern1);
par2 = kernExtractParam(lfmKern2);
if (par1==par2)
    autoCorr = true;
else
    autoCorr = false;
end;
autoCorr = false;
% Parameters of the simulation (in the order providen by kernExtractParam)

m = [lfmKern1.mass lfmKern2.mass];                  % Par. 1
D = [lfmKern1.spring lfmKern2.spring];              % Par. 2
C = [lfmKern1.damper lfmKern2.damper];              % Par. 3
sigma2 = 2/lfmKern1.inverseWidth;                   % Par. 4
sigma = sqrt(sigma2);
S = [lfmKern1.sensitivity lfmKern2.sensitivity];    % Par. 5

alpha = C./(2*m);
omega = sqrt(D./m-alpha.^2);

% Initialization of vectors and matrices

T1 = size(t1, 1);
T2 = size(t2, 1);
matGrad = zeros(T1, T2);

g1 = zeros(1,5);
g2 = zeros(1,5);

% Gradient with respect to m, D and C

for ind_theta = 1:3 % Parameter (m, D or C)
  for ind_par = 0:1 % System (1 or 2)

    % Choosing the right gradients for m, omega, gamma1 and gamma2

    switch ind_theta
        case 1  % Gradient wrt m
            gradThetaM = [1-ind_par ind_par];
            gradThetaAlpha = -C./(2*(m.^2));
            gradThetaOmega = (C.^2-2*m.*D)./(2*(m.^2).*sqrt(4*m.*D-C.^2));
        case 2  % Gradient wrt D
            gradThetaM = zeros(1,2);
            gradThetaAlpha = zeros(1,2);
            gradThetaOmega = 1./sqrt(4*m.*D-C.^2);
        case 3  % Gradient wrt C
            gradThetaM = zeros(1,2);
            gradThetaAlpha = 1./(2*m);
            gradThetaOmega = -C./(2*m.*sqrt(4*m.*D-C.^2));
    end
  
    gradThetaGamma1 = gradThetaAlpha + j*gradThetaOmega;
    gradThetaGamma2 = gradThetaAlpha - j*gradThetaOmega;
  
    % Gradient evaluation
    
    if isreal(omega)
        gamma1 = alpha(1) + j*omega(1);
        gamma2 = alpha(2) + j*omega(2);

        if autoCorr
            matInd = ones(4,2);
        else
            matInd = repmat([ind_par 1-ind_par; 1-ind_par ind_par],2,1);
        end;
        
        gradThetaGamma = [conj(gradThetaGamma1(2)), gradThetaGamma1(1); ...
            gradThetaGamma1(1), conj(gradThetaGamma1(2)); ...
            gradThetaGamma1(2), gradThetaGamma1(1); ...
            gradThetaGamma1(1), gradThetaGamma1(2)].*matInd;
        matGrad = (sigma*prod(S)*sqrt(pi)/(4*prod(m)*prod(omega))) ...
                * real(lfmGradientH(conj(gamma2),gamma1,sigma2, ...
                    gradThetaGamma(1,:),t1,t2) ...
                + lfmGradientH(gamma1,conj(gamma2),sigma2, ...
                    gradThetaGamma(2,:),t2,t1) ...
                - lfmGradientH(gamma2,gamma1,sigma2, ...
                    gradThetaGamma(3,:),t1,t2) ...
                - lfmGradientH(gamma1,gamma2,sigma2, ...
                    gradThetaGamma(4,:),t2,t1) ...
                - (1+autoCorr)*(gradThetaM(1+ind_par)/m(1+ind_par) ...
                + gradThetaOmega(1+ind_par)/omega(1+ind_par)) ...
                * (lfmComputeH(conj(gamma2),gamma1,sigma2,t1,t2) ...
                + lfmComputeH(gamma1,conj(gamma2),sigma2,t2,t1) ...
                - lfmComputeH(gamma2,gamma1,sigma2,t1,t2) ...
                - lfmComputeH(gamma1,gamma2,sigma2,t2,t1)));
    else
        gamma1_p = alpha(1) + j*omega(1);
        gamma1_m = alpha(1) - j*omega(1);
        gamma2_p = alpha(2) + j*omega(2);
        gamma2_m = alpha(2) - j*omega(2);
        
        if autoCorr
            matInd = ones(8,2);
        else
            matInd = repmat([ind_par 1-ind_par; 1-ind_par ind_par],4,1);
        end;

        gradThetaGamma = [gradThetaGamma2(2) gradThetaGamma1(1); ...
            gradThetaGamma1(1) gradThetaGamma2(2); ...
            gradThetaGamma1(2) gradThetaGamma2(1); ...
            gradThetaGamma2(1) gradThetaGamma1(2); ...
            gradThetaGamma2(2) gradThetaGamma2(1); ...
            gradThetaGamma2(1) gradThetaGamma2(2); ...
            gradThetaGamma1(2) gradThetaGamma1(1); ...
            gradThetaGamma1(1) gradThetaGamma1(2)].*matInd;
        matGrad = (sigma*prod(S)*sqrt(pi)/(8*prod(m)*prod(omega))) ...
                * (lfmGradientH(gamma2_m,gamma1_p,sigma2, ...
                    gradThetaGamma(1,:),t1,t2) ...
                + lfmGradientH(gamma1_p,gamma2_m,sigma2, ...
                    gradThetaGamma(2,:),t2,t1) ...
                + lfmGradientH(gamma2_p,gamma1_m,sigma2, ...
                    gradThetaGamma(3,:),t1,t2) ...
                + lfmGradientH(gamma1_m,gamma2_p,sigma2, ...
                    gradThetaGamma(4,:),t2,t1) ...
                - lfmGradientH(gamma2_m,gamma1_m,sigma2, ...
                    gradThetaGamma(5,:),t1,t2) ...
                - lfmGradientH(gamma1_m,gamma2_m,sigma2, ...
                    gradThetaGamma(6,:),t2,t1) ...
                - lfmGradientH(gamma2_p,gamma1_p,sigma2, ...
                    gradThetaGamma(7,:),t1,t2) ...
                - lfmGradientH(gamma1_p,gamma2_p,sigma2, ...
                    gradThetaGamma(8,:),t2,t1) ...
                - (1+autoCorr)*(gradThetaM(1+ind_par)/m(1+ind_par) ...
                + gradThetaOmega(1+ind_par)/omega(1+ind_par)) ...
                * (lfmComputeH(gamma2_m,gamma1_p,sigma2,t1,t2) ...
                + lfmComputeH(gamma1_p,gamma2_m,sigma2,t2,t1) ...
                + lfmComputeH(gamma2_p,gamma1_m,sigma2,t1,t2) ...
                + lfmComputeH(gamma1_m,gamma2_p,sigma2,t2,t1) ...
                - lfmComputeH(gamma2_m,gamma1_m,sigma2,t1,t2) ...
                - lfmComputeH(gamma1_m,gamma2_m,sigma2,t2,t1) ...
                - lfmComputeH(gamma2_p,gamma1_p,sigma2,t1,t2) ...
                - lfmComputeH(gamma1_p,gamma2_p,sigma2,t2,t1)));
    end
    % Check the parameter to assign the derivative
    if ind_par == 0
      g1(ind_theta) = sum(sum(matGrad.*covGrad));
    else
      g2(ind_theta) = sum(sum(matGrad.*covGrad));
    end
    % If we are evaluating an autocorrelation there is no need to obtain
    % the second derivative
    if autoCorr
        g2(ind_theta) = g1(ind_theta);
        break;
    end;
  end
end

% Gradients with respect to sigma

if isreal(omega)
    gamma1 = alpha(1) + j*omega(1);
    gamma2 = alpha(2) + j*omega(2);
    matGrad = (prod(S)*sqrt(pi)/(4*prod(m)*prod(omega))) ...
        * real(lfmComputeH(conj(gamma2),gamma1,sigma2,t1,t2) ...
        + lfmComputeH(gamma1,conj(gamma2),sigma2,t2,t1) ...
        - lfmComputeH(gamma2,gamma1,sigma2,t1,t2) ...
        - lfmComputeH(gamma1,gamma2,sigma2,t2,t1) ...
        + sigma*(lfmGradientSigmaH(conj(gamma2),gamma1,sigma2,t1,t2) ...
        + lfmGradientSigmaH(gamma1,conj(gamma2),sigma2,t2,t1) ...
        - lfmGradientSigmaH(gamma2,gamma1,sigma2,t1,t2) ...
        - lfmGradientSigmaH(gamma1,gamma2,sigma2,t2,t1)));
else
    gamma1_p = alpha(1) + j*omega(1);
    gamma1_m = alpha(1) - j*omega(1);
    gamma2_p = alpha(2) + j*omega(2);
    gamma2_m = alpha(2) - j*omega(2);
    matGrad = (prod(S)*sqrt(pi)/(8*prod(m)*prod(omega))) ...
       * (lfmComputeH(gamma2_m,gamma1_p,sigma2,t1,t2) ...
       + lfmComputeH(gamma1_p,gamma2_m,sigma2,t2,t1) ...
       + lfmComputeH(gamma2_p,gamma1_m,sigma2,t1,t2) ...
       + lfmComputeH(gamma1_m,gamma2_p,sigma2,t2,t1) ...
       - lfmComputeH(gamma2_m,gamma1_m,sigma2,t1,t2) ...
       - lfmComputeH(gamma1_m,gamma2_m,sigma2,t2,t1) ...
       - lfmComputeH(gamma2_p,gamma1_p,sigma2,t1,t2) ...
       - lfmComputeH(gamma1_p,gamma2_p,sigma2,t2,t1) ...
       + sigma*(lfmGradientSigmaH(gamma2_m,gamma1_p,sigma2,t1,t2) ...
       + lfmGradientSigmaH(gamma1_p,gamma2_m,sigma2,t2,t1) ...
       + lfmGradientSigmaH(gamma2_p,gamma1_m,sigma2,t1,t2) ...
       + lfmGradientSigmaH(gamma1_m,gamma2_p,sigma2,t2,t1) ...
       - lfmGradientSigmaH(gamma2_m,gamma1_m,sigma2,t1,t2) ...
       - lfmGradientSigmaH(gamma1_m,gamma2_m,sigma2,t2,t1) ...
       - lfmGradientSigmaH(gamma2_p,gamma1_p,sigma2,t1,t2) ...
       - lfmGradientSigmaH(gamma1_p,gamma2_p,sigma2,t2,t1)));
end;

g1(4) = sum(sum(matGrad.*covGrad))*(-(sigma^3)/4);
g2(4) = g1(4);

% Gradients with respect to S

if isreal(omega)
    gamma1 = alpha(1) + j*omega(1);
    gamma2 = alpha(2) + j*omega(2);
    matGrad = (sigma*sqrt(pi)/(4*prod(m)*prod(omega))) ...
            * real(lfmComputeH(conj(gamma2),gamma1,sigma2,t1,t2) ...
        + lfmComputeH(gamma1,conj(gamma2),sigma2,t2,t1) ...
            - lfmComputeH(gamma2,gamma1,sigma2,t1,t2) ...
        - lfmComputeH(gamma1,gamma2,sigma2,t2,t1));
else
    gamma1_p = alpha(1) + j*omega(1);
    gamma1_m = alpha(1) - j*omega(1);
    gamma2_p = alpha(2) + j*omega(2);
    gamma2_m = alpha(2) - j*omega(2);
    matGrad = (sigma*sqrt(pi)/(8*prod(m)*prod(omega))) ...
           * (lfmComputeH(gamma2_m,gamma1_p,sigma2,t1,t2) ...
       + lfmComputeH(gamma1_p,gamma2_m,sigma2,t2,t1) ...
           + lfmComputeH(gamma2_p,gamma1_m,sigma2,t1,t2) ...
       + lfmComputeH(gamma1_m,gamma2_p,sigma2,t2,t1) ...
           - lfmComputeH(gamma2_m,gamma1_m,sigma2,t1,t2) ...
       - lfmComputeH(gamma1_m,gamma2_m,sigma2,t2,t1) ...
           - lfmComputeH(gamma2_p,gamma1_p,sigma2,t1,t2) ...
       - lfmComputeH(gamma1_p,gamma2_p,sigma2,t2,t1));
end;

g1(5) = sum(sum((1+autoCorr)*S(2)*matGrad.*covGrad));
if autoCorr
    g2(5) = g1(5);
else
    g2(5) = sum(sum(S(1)*matGrad.*covGrad));
end
g2(4) = 0; % Otherwise is counted twice, temporarly changed by MA
return