function K = simXrbfKernCompute(simKern, rbfKern, t1, t2)

% SIMXRBFKERNCOMPUTE Compute a cross kernel between the SIM and RBF kernels.
%
%	Description:
%
%	K = SIMXRBFKERNCOMPUTE(SIMKERN, RBFKERN, T) computes cross kernel
%	terms between SIM and RBF kernels for the multiple output kernel.
%	 Returns:
%	  K - block of values from kernel matrix.
%	 Arguments:
%	  SIMKERN - the kernel structure associated with the SIM kernel.
%	  RBFKERN - the kernel structure associated with the RBF kernel.
%	  T - inputs for which kernel is to be computed.
%
%	K = SIMXRBFKERNCOMPUTE(SIMKERN, RBFKERN, T1, T2) computes cross
%	kernel terms between SIM and RBF kernels for the multiple output
%	kernel.
%	 Returns:
%	  K - block of values from kernel matrix.
%	 Arguments:
%	  SIMKERN - the kernel structure associated with the SIM kernel.
%	  RBFKERN - the kernel structure associated with the RBF kernel.
%	  T1 - row inputs for which kernel is to be computed.
%	  T2 - column inputs for which kernel is to be computed.
%	
%
%	See also
%	MULTIKERNPARAMINIT, MULTIKERNCOMPUTE, SIMKERNPARAMINIT, RBFKERNPARAMINIT


%	Copyright (c) 2006 Neil D. Lawrence
% 	simXrbfKernCompute.m CVS version 1.3
% 	simXrbfKernCompute.m SVN version 1
% 	last update 2007-12-19T07:05:33.000000Z

if nargin < 4
  t2 = t1;
end
if size(t1, 2) > 1 | size(t2, 2) > 1
  error('Input can only have one column');
end
if simKern.inverseWidth ~= rbfKern.inverseWidth
  error('Kernels cannot be cross combined if they have different inverse widths.')
end
dim1 = size(t1, 1);
dim2 = size(t2, 1);
t1 = t1 - simKern.delay;
t1Mat = repmat(t1, [1 dim2]);
t2Mat = repmat(t2', [dim1 1]);
diffT = (t1Mat - t2Mat);
sigma = sqrt(2/simKern.inverseWidth);

invSigmaDiffT = 1/sigma*diffT;
halfSigmaD_i = 0.5*sigma*simKern.decay;

%lnPart1 = log(erf(invSigmaDiffT - halfSigmaD_i) + erf(t2Mat/sigma + halfSigmaD_i));
lnPart1 = zeros(size(t1Mat));
warnState = warning('query', 'MATLAB:log:logOfZero');
warning('off', 'MATLAB:log:logOfZero');
I1 = sign(real(halfSigmaD_i - invSigmaDiffT)) ...
    ~= sign(real(halfSigmaD_i + t2Mat/sigma));
I2 = (real(halfSigmaD_i - invSigmaDiffT) > 0) & ~I1;
I3 = ~I2 & ~I1;
lnPart1(I1) = log(  erfc( real(halfSigmaD_i - invSigmaDiffT(I1))) ...
		   - erfc( real(halfSigmaD_i + t2Mat(I1)/sigma)));
%lnPart1(I2) = log(  erfc( real(halfSigmaD_i - invSigmaDiffT(I2))) ...
%		 - erfc( real(halfSigmaD_i + t2Mat(I2)/sigma)));
%lnPart1(I3) = log(- erfc(-real(halfSigmaD_i - invSigmaDiffT(I3))) ...
%		 + erfc(-real(halfSigmaD_i + t2Mat(I3)/sigma)));
lnPart1(I2) = log(  erfcx( real(halfSigmaD_i - invSigmaDiffT(I2))) ...
		   - exp(  real(halfSigmaD_i - invSigmaDiffT(I2)).^2 ...
			   - real(halfSigmaD_i + t2Mat(I2)/sigma).^2) ...
		   .* erfcx( real(halfSigmaD_i + t2Mat(I2)/sigma))) ...
    - real(halfSigmaD_i - invSigmaDiffT(I2)).^2;
lnPart1(I3) = log(- exp(real(halfSigmaD_i + t2Mat(I3)/sigma).^2 ...
		       - real(halfSigmaD_i - invSigmaDiffT(I3)).^2) ...
		 .* erfcx(-real(halfSigmaD_i - invSigmaDiffT(I3))) ...
		 + erfcx(-real(halfSigmaD_i + t2Mat(I3)/sigma))) ...
    - real(halfSigmaD_i + t2Mat(I3)/sigma).^2;
warning(warnState.state, 'MATLAB:log:logOfZero');
K = exp(halfSigmaD_i*halfSigmaD_i - simKern.decay*diffT + lnPart1);

K = 0.5*sqrt(simKern.variance)*sqrt(rbfKern.variance)*K*sqrt(pi)*sigma;
